OBJ
''#################################################################################################
''                              Object declarations                               
''#################################################################################################
 tv    : "tv"
 gr    : "graphics"
 Num   : "Numbers"
Freq   : "Synth"  

CON     ''This CON block belongs to the Frequency Synthesizer

OutputFrequencyPin = 0

       Frequency_A = 1400
       Frequency_B = 1600

CON     ''This CON block belongs to the Spectrum Analyzer moved up from below

 InputPin = 0
  Samples = 25  'Each Sample contains 8-Periods


StartFrequency  =1200
StopFrequency = 1800
StepFrequency   = 10       

CON
''#################################################################################################
''                              Setup Constants for Propeller Board                              
''#################################################################################################
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

CON
''#################################################################################################
''                         Setup Constants for Graphics Configuration
''#################################################################################################
  _stack = ($3000 + $3000) >> 2
  x_tiles = 16
  y_tiles = 12
  paramcount = 14       
  bitmap_base = $2000
  display_base = $5000
VAR
''#################################################################################################
''                           Variables used in Graphics Configuration
''#################################################################################################
  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

VAR
''#################################################################################################
''                           Variables used in TV DEMO
''#################################################################################################

  long  ScreenWidthBig,ScreenWidthSmall
  
PUB TV_DEMO | i, FreqTest,dx,dy,Scale,SpectralMAX
''#################################################################################################
''                           TV Audio Spectrum Analyzer DEMO main Program
''#################################################################################################

  Freq.Synth("A",OutputFrequencyPin, Frequency_A)     'Start Freqency Synthesizer for Frequency "A"
  Freq.Synth("B",OutputFrequencyPin, Frequency_B)     'Start Freqency Synthesizer for Frequency "B"

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  longfill (@colors,$2B069C92,64)

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 128, 96, bitmap_base)

  repeat

    'clear bitmap
    gr.clear
    gr.color(2)
    gr.textmode(1, 1, 6, %0101)
    gr.text(0, 91, String("Audio Spectrum Analyzer"))
    gr.textmode(1, 1, 6, %1000)
    gr.text(-100,-80, Num.ToStr(StartFrequency,10))
    gr.textmode(1, 1, 6, %0100)
    gr.text(100,-80, Num.ToStr(StopFrequency,10))
    gr.textmode(1, 1, 6, %0100)
    gr.text(0,-80, Num.ToStr((StopFrequency+StartFrequency)/2,10))
    gr.textmode(1, 1, 6, %1000)
    gr.text(-100,-95, Num.ToStr(StepFrequency,10))
    gr.text(0, -95, String("Hz Sweep Steps"))


    if (StopFrequency - StartFrequency) > 200
        ScreenWidthBig := (StopFrequency - StartFrequency)/200
        ScreenWidthSmall := 1
    else
        ScreenWidthBig := 1000
        ScreenWidthSmall := 200000/(StopFrequency - StartFrequency)

        

    gr.plot(-100,-50)
    repeat FreqTest from StartFrequency to StopFrequency step StepFrequency
      'TestFrequency := 1000                               'Frequency to detect
      Detect(FreqTest)          'Acquire and accumulate Signal level at desired frequency

      SpectralMAX := Samples * 32
      dy := (DataResult * 100)/ SpectralMAX
      
      dx := (((FreqTest - StartFrequency)*ScreenWidthSmall)/ScreenWidthBig)-100
      
      gr.line(dx,dy-50)

      'copy bitmap to display
      gr.copy(display_base)
    repeat    

DAT
''#################################################################################################
''                             Graphics Display Configuration
''#################################################################################################
tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    50_000_000'_xinfreq<<4  'broadcast
                        long    0               'auralcog
CON
{{
******************************************************************************************************************
******************************************************************************************************************
        Everything below this CON block is all you need for the Spectrum Analyzer Frequency Detection portion.
******************************************************************************************************************
******************************************************************************************************************
}}
CON
{{  'Moved up to top for easier debugging....
 InputPin = 0
  Samples = 10  'Each Sample contains 8-Periods


StartFrequency  = 800
StopFrequency = 1_100
StepFrequency   = 5
}}
VAR

long    DataResult

PUB Detect(SpecificFrequency)|Status
    Status~                                             'clear status register                       
    _Status := @Status                                  'Store address of Status for Assembly routine
    cps := clkfreq/SpecificFrequency #> 40              'Calculate Clocks Per Sample                 
    cognew(@FrequencyDetect,@DataResult)                'Start a new instance of FastIO              
    repeat while Status == 0                            'Wait here until samples are complete
    
DAT
FrequencyDetect         org    0
              mov       temp, #1        wz      'Configure Pin as an Input
              shl       temp, InputPinMask
              mov       InputPinMask, temp
              muxz      dira, InputPinMask      'Set Pin to an INPUT

              mov       acc1, #0
              mov       acc2, #0

              shr       cps, #2

              mov       temp, cnt               'grab current counter value

NewSample     add       temp, cps
              add       temp, cps
              add       temp, cps
              add       temp, cps
'--------------------------------------------------------------------------------------------------- 1-Period
              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_c  add       acc1, #1
        if_nc sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag        
        if_c  add       acc2, #1
        if_nc sub       acc2, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc1, #1
        if_c  sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc2, #1
        if_c  sub       acc2, #1
'--------------------------------------------------------------------------------------------------- 2-Periods        
              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_c  add       acc1, #1
        if_nc sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag        
        if_c  add       acc2, #1
        if_nc sub       acc2, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc1, #1
        if_c  sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc2, #1
        if_c  sub       acc2, #1
'--------------------------------------------------------------------------------------------------- 3-Periods        
              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_c  add       acc1, #1
        if_nc sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag        
        if_c  add       acc2, #1
        if_nc sub       acc2, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc1, #1
        if_c  sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc2, #1
        if_c  sub       acc2, #1
'--------------------------------------------------------------------------------------------------- 4-Periods        
              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_c  add       acc1, #1
        if_nc sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag        
        if_c  add       acc2, #1
        if_nc sub       acc2, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc1, #1
        if_c  sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc2, #1
        if_c  sub       acc2, #1
'--------------------------------------------------------------------------------------------------- 5-Periods        
              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_c  add       acc1, #1
        if_nc sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag        
        if_c  add       acc2, #1
        if_nc sub       acc2, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc1, #1
        if_c  sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc2, #1
        if_c  sub       acc2, #1
'--------------------------------------------------------------------------------------------------- 6-Periods        
              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_c  add       acc1, #1
        if_nc sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag        
        if_c  add       acc2, #1
        if_nc sub       acc2, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc1, #1
        if_c  sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc2, #1
        if_c  sub       acc2, #1
'--------------------------------------------------------------------------------------------------- 7-Periods        
              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_c  add       acc1, #1
        if_nc sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag        
        if_c  add       acc2, #1
        if_nc sub       acc2, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc1, #1
        if_c  sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc2, #1
        if_c  sub       acc2, #1
'--------------------------------------------------------------------------------------------------- 8-Periods        
              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_c  add       acc1, #1
        if_nc sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag        
        if_c  add       acc2, #1
        if_nc sub       acc2, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc1, #1
        if_c  sub       acc1, #1

              waitcnt   temp, cps               'Wait for next time interval
              test      InputPinMask, ina wc    'Check I/O pin for a "1" or a "0" ; set "C" flag
        if_nc add       acc2, #1
        if_c  sub       acc2, #1
'------------------------------------------------------------------------------------------------
              djnz      DataSamples, #NewSample 'Get next sample

              abs       acc1, acc1
              abs       acc2, acc2

              add       acc1, acc2              

              wrlong    acc1, par               'Write the accumulator value to the Data pointer

              mov       temp, #1                'Update 'Status' register indicating that data is ready
              wrlong    temp, _Status

              cogid     temp                    
              cogstop   temp                    'STOP this COG

InputPinMask  long      InputPin
DataSamples   long      Samples
cps           long      0
_Status       long      0
acc1          long      0
acc2          long      0
temp          long      0                                             